import os
import oauth2 as oauth
import urllib
from django.conf import settings

class TradeKingAPI(object):
    
    HOST = 'https://api.tradeking.com/v1'

    def __init__(self,
                 response_format='json'):

        oAuthConsumer = oauth.Consumer(
            key=settings.TK_CONSUMER_KEY,
            secret=settings.TK_CONSUMER_SECRET)

        oAuthToken = oauth.Token(
            key=settings.TK_OAUTH_TOKEN,
            secret=settings.TK_OAUTH_SECRET)

        self.oAuthRequest = oauth.Client(oAuthConsumer, oAuthToken)
        self.format = response_format

#~------------------------------------------------------------------------------
#~Account Calls ----------------------------------------------------------------

    def accounts(self):
        """This call will return detailed balance and holding information for each
        account associated with a user.
        """
        url = '{0}/accounts.{1}'.format(self.HOST, self.format)
        response, content = self.oAuthRequest.request(url)
        return response, content, url

    def accounts_balances(self):
        """This call will return summary balance information for each account
        associated with a user as well as the total value for all accounts
        associated with a user.
        """
        url = '{0}/accounts/balances.{1}'.format(self.HOST, self.format)
        response, content = self.oAuthRequest.request(url)
        return response, content, url

    def accounts_id(self, id):
        """This call will return detailed balance and holding information for the
        account number specified in the URI.
        
        param id: id of the account
        """
        url = '{0}/accounts/{1}.{2}'.format(self.HOST, id, self.format)
        response, content = self.oAuthRequest.request(url)
        return response, content, url

    def accounts_id_balances(self, id):
        """This call will return detailed balance information for the account
        number specified in the URI.

        param id: id of the account        
        """
        url = '{0}/accounts/{1}/balances.{2}'.format(self.HOST, id, self.format)
        response, content = self.oAuthRequest.request(url)
        return response, content, url

    def accounts_id_history(self, id, range='all', transactions='all'):
        """This call will return account activity for the account number specified
        in the URI. This call supports optional date range or transaction type
        filters.

        param id: id of the account
        param range: all, today, current_week, current_month, last_month
        param transactions: all, bookkeeping, trade
        """
        payload = urllib.urlencode(dict(range=range, transactions=transactions))
        url = '{0}/accounts/{1}/history.{2}?{3}'.format(self.HOST, id, self.format, payload)
        response, content = self.oAuthRequest.request(url)
        return response, content, url, payload

    def accounts_id_holdings(self, id):
        url = '{0}/accounts/{1}/holdings.{2}'.format(self.HOST, id, self.format)
        response, content = self.oAuthRequest.request(url)
        return response, content, url

#~------------------------------------------------------------------------------
#~Trade Calls ------------------------------------------------------------------

    def accounts_id_orders(self, id, method='GET', fixml=None, headers=None):
        url = '{0}/accounts/{1}/orders.{2}'.format(self.HOST, id, self.format)
        if method.lower() == 'get':
            response, content = self.oAuthRequest.request(url)
            return response, content, url
        elif method.lower() == 'post':
            response, content = self.oAuthRequest.request(url, body=fixml, headers=headers, method=method)
            return response, content, url, fixml, headers
        else:
            raise "Invalid method {}".format(method)

    def accounts_id_orders_preview(self, id, fixml):
        url = '{0}/accounts/{1}/orders/preview.{2}'.format(self.HOST, id, self.format)
        response, content = self.oAuthRequest.request(url, body=fixml, method='POST')
        return response, content, url, fixml

#~------------------------------------------------------------------------------
#~Market Calls -----------------------------------------------------------------

    def market_clock(self):
        url = '{0}/market/clock.{1}'.format(self.HOST, self.format)
        response, content = self.oAuthRequest.request(url)
        return response, content, url

    def market_ext_quotes(self, symbols, fids=''):
        payload = urllib.urlencode(dict(symbols=symbols, fids=fids))
        url = '{0}/market/ext/quotes.{1}?{2}'.format(self.HOST, self.format, payload)
        response, content = self.oAuthRequest.request(url)
        return response, content, url, payload

    def market_historical_search(self, symbols, interval, startdate, enddate):
        payload = urllib.urlencode(dict(symbols   = symbols, 
                                        interval  = interval,
                                        startdate = startdate,
                                        enddate   = enddate))
        url = '{0}/market/historical/search.{1}?{2}'.format(self.HOST, self.format, payload)
        response, content = self.oAuthRequest.request(url)
        return response, content, url, payload

    def market_news_search(self, keywords, startdate, enddate, symbols=None, maxhits=10):
        payload = urllib.urlencode(dict(keywords  = keywords, 
                       symbols   = symbols, 
                       maxhits   = maxhits,
                       startdate = startdate, 
                       enddate   = enddate))
        url = '{0}/market/news/search.{1}?{2}'.format(self.HOST, self.format, payload)
        response, content = self.oAuthRequest.request(url)
        return response, content, url, payload

    def market_news_id(self, id):
        url = '{0}/market/news/{1}.{2}'.format(self.HOST, id, self.format)
        response, content = self.oAuthRequest.request(url)
        return response, content, url

    def market_options_search(self, symbol, query=None, fids=None):
        payload = urllib.urlencode(dict(symbol=symbol, query=query, fids=fids))
        url = '{0}/market/options/search.{1}?{2}'.format(self.HOST, self.format, payload)
        response, content = self.oAuthRequest.request(url)
        return response, content, url, payload

    def market_options_strikes(self, symbol):
        payload = urllib.urlencode(dict(symbol=symbol))
        url = '{0}/market/options/strikes.{1}?{2}'.format(self.HOST, self.format, payload)
        response, content = self.oAuthRequest.request(url)
        return response, content, url, payload

    def market_options_expirations(self, symbol):
        payload = urllib.urlencode(dict(symbol=symbol))
        url = '{0}/market/options/expirations.{1}?{2}'.format(self.HOST, self.format, payload)
        response, content = self.oAuthRequest.request(url)
        return response, content, url, payload

    def market_timesales(self, symbol):
        payload = urllib.urlencode(dict(symbol=symbol))
        url = '{0}/market/options/expirations.{1}?{2}'.format(self.HOST, self.format, payload)
        response, content = self.oAuthRequest.request(url)
        return response, content, url, payload

    def market_toplists(self, toplist, exchange='N'):
        """
        Exchange abbreviations: (the default exchange is NYSE)
        A:  American Stock Exchange
        N:  New York Stock Exchange
        Q:  NASDAQ
        U:  NASDAQ Bulletin Board
        V:  NASDAQ OTC Other

        Available list types include:
        toplosers       Top losers by dollar amount
        toppctlosers    Top percentage losers
        topvolume       Top volume
        topactive       Top active
        topgainers      Top gainers by dollar amount
        toppctgainers   Top percentage gainers
        """
        payload = urllib.urlencode(dict(exchange=exchange))
        url = '{0}/market/toplists/{1}.{2}?{3}'.format(self.HOST, toplist, self.format, payload)
        response, content = self.oAuthRequest.request(url)
        return response, content, url, payload

#~------------------------------------------------------------------------------
#~Member Calls -----------------------------------------------------------------
    def member_profile(self):
        url = '{0}/member/profile.{1}'.format(self.HOST, self.format)
        response, content = self.oAuthRequest.request(url)
        return response, content, url
#~------------------------------------------------------------------------------
#~Utility Calls ----------------------------------------------------------------
    def utility_status(self):
        url = '{0}/utility/status.{1}'.format(self.HOST, self.format)
        response, content = self.oAuthRequest.request(url)
        return response, content, url

    def utility_version(self):
        url = '{0}/utility/version.{1}'.format(self.HOST, self.format)
        response, content = self.oAuthRequest.request(url)
        return response, content, url
#~------------------------------------------------------------------------------
#~Watchlist Calls --------------------------------------------------------------
    def watchlists(self, name=None, symbols=None, method='GET'):
        if method.lower() == 'post':
            if name is None:
                raise "Need name for watchlist"
            elif symbols is None:
                raise "Need symbols to POST"
            else:
                payload = urllib.urlencode(dict(symbols=symbols, id=name))
                url = '{0}/watchlists.{1}?{2}'.format(self.HOST, self.format, payload)
                response, content = self.oAuthRequest.request(url)
                return response, content, url, payload
        elif method.lower() == 'get':
            url = '{0}/watchlists.{1}'.format(self.HOST, self.format)
            response, content = self.oAuthRequest.request(url)
            return response, content, url
        else:
            raise 'Invalid method {}'.format(method)

    def watchlists_id(self, name, method='GET'):
        url = '{0}/watchlists/{1}.{2}'.format(self.HOST, name, self.format)
        if method.lower() == 'get':
            response, content = self.oAuthRequest.request(url)
            return response, content, url
        elif method.lower() == 'delete':
            return self.client.delete(url)
        else:
            raise 'Invalid method {}'.format(method)

    def watchlists_id_symbols(self, name, symbols, method='POST'):
        """
        [POST] - add the symbols in the form parameters to the watchlist
                 specified in the URI

        [DELETE] - delete the symbol in the URI for the watchlist specified in
                   the URI
        """
        url = '{0}/watchlists/{1}/symbols.{2}'.format(self.HOST, name, self.format)
        if method.lower() == 'post':
            response, content = self.oAuthRequest.request(url)
            return response, content, url
        elif method.lower() == 'delete':
            return self.client.delete(url)
        else:
            raise 'Invalid method {}'.format(method)
#~------------------------------------------------------------------------------



if __name__ == "__main__":
    api = TradeKingAPI()

    print('accounts_id()')
    r = api.accounts_id('38728224')
    print r
    
    print('accounts()')
    r = api.accounts()
    print r
     
    print('accounts_balances()')
    r = api.accounts_balances()
    print r